@@ -2,24 +2,22 @@ import itertools
 import logging
 from functools import total_ordering
 
+import autonetkit
 import autonetkit.log as log
 from autonetkit.anm.interface import NmPort
 from autonetkit.log import CustomAdapter
+from autonetkit.anm.ank_element import AnkElement
+
 
 
 @total_ordering
-class NmNode(object):
+class NmNode(AnkElement):
 
     """NmNode"""
 
-    def __init__(
-        self,
-        anm,
-        overlay_id,
-        node_id,
-    ):
+    def __init__(self, anm, overlay_id, node_id):
 
-# Set using this method to bypass __setattr__
+        # Set using this method to bypass __setattr__
 
         object.__setattr__(self, 'anm', anm)
         object.__setattr__(self, 'overlay_id', overlay_id)
@@ -30,6 +28,7 @@ class NmNode(object):
         #logger = CustomAdapter(logger, {'item': logstring})
         logger = log
         object.__setattr__(self, 'log', logger)
+        self.init_logging("node")
 
     def __hash__(self):
         """"""
@@ -37,20 +36,68 @@ class NmNode(object):
         return hash(self.node_id)
 
     def __nonzero__(self):
-        """Allows for checking if node exists"""
+        """Allows for checking if node exists
+
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> bool(r1)
+        True
+        >>> r_test = anm['phy'].node("test")
+        >>> bool(r_test)
+        False
+
+        """
 
         return self.node_id in self._graph
 
     def __iter__(self):
-        """Shortcut to iterate over the physical interfaces of this node"""
+        """Shortcut to iterate over the physical interfaces of this node
 
-        return self.interfaces(category='physical')
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> list(r1)
+        [eth0.r1, eth1.r1]
+
+
+        """
+
+        return iter(self.interfaces(category='physical'))
 
     def __len__(self):
-        return len(self.__iter())
+        """Number of phyiscal interfaces of a node
+
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> len(r1)
+        2
+        >>> r2 = anm['phy'].node("r2")
+        >>> len(r2)
+        3
+
+
+        """
+        return len(list(self.__iter__()))
 
     def __eq__(self, other):
-        """"""
+        """
+
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> rb = anm['phy'].node("r1")
+        >>> r1 == rb
+        True
+        >>> r2 = anm['phy'].node("r2")
+        >>> r1 == r2
+        False
+
+        Can also compare to a label
+
+        >>> r1 == "r1"
+        True
+        >>> r1 == "r2"
+        False
+
+        """
 
         try:
             return self.node_id == other.node_id
@@ -58,34 +105,62 @@ class NmNode(object):
             return self.node_id == other  # eg compare Node to label
 
     def __ne__(self, other):
+        """
+
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> r2 = anm['phy'].node("r2")
+        >>> r1 != r2
+        True
+
+
+        """
         return not self.__eq__(other)
 
     @property
     def loopback_zero(self):
-        """"""
+        """
+
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> lo_zero = r1.loopback_zero
+
+        """
+
+        # TODO: initialise id for loopback zero?
+        # TODO: Set category for loopback zero to be "loopback"
 
         return (i for i in self.interfaces('is_loopback_zero')).next()
 
-    def physical_interfaces(self):
+    def physical_interfaces(self, *args, **kwargs):
         """"""
+        kwargs['category'] = "physical"
+        return self.interfaces(*args, **kwargs)
 
-        return self.interfaces(category='physical')
-
-    def loopback_interfaces(self):
+    def loopback_interfaces(self, *args, **kwargs):
+        #TODO: allow abiility to skip loopback zero
         """"""
-
-        return self.interfaces(category='loopback')
+        kwargs['category'] = "loopback"
+        return self.interfaces(*args, **kwargs)
 
     def is_multigraph(self):
         return self._graph.is_multigraph()
 
     def __lt__(self, other):
-        """"""
+        """
+
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> r2 = anm['phy'].node("r2")
+        >>> r1 < r2
+        True
+
+        """
 
 # want [r1, r2, ..., r11, r12, ..., r21, r22] not [r1, r11, r12, r2, r21, r22]
 # so need to look at numeric part
         import string
-        #TODO: use human sort from StackOverflow
+        # TODO: use human sort from StackOverflow
 
         # sort on label if available
         if self.label is not None:
@@ -136,12 +211,8 @@ class NmNode(object):
 
     # TODO: interface function access needs to be cleaned up
 
-    def _add_interface(
-        self,
-        description=None,
-        category='physical',
-        **kwargs
-    ):
+    def _add_interface(self, description=None, category='physical',
+                       **kwargs):
         """"""
 
         data = dict(kwargs)
@@ -149,7 +220,7 @@ class NmNode(object):
         if self.overlay_id != 'phy' and self['phy']:
             next_id = self['phy']._next_int_id()
             self['phy']._ports[next_id] = {'category': category,
-                                             'description': description}
+                                           'description': description}
 
             # TODO: fix this workaround for not returning description from phy
             # graph
@@ -163,20 +234,56 @@ class NmNode(object):
         self._ports[next_id] = data
         return next_id
 
+    def _sync_loopbacks(self, interface_id):
+        """Syncs a newly added loopback across all overlays the
+        node is in"""
+        for overlay in self.anm:
+            if self not in overlay:
+                continue
+
+            if overlay._overlay_id in ("phy", self.overlay_id):
+                # don't copy to self or phy as already there
+                continue
+
+            if overlay._overlay_id in ("graphics"):
+                """
+                TODO: debug why get problem for graphics
+                 File "/autonetkit/autonetkit/anm/node.py", line 257, in _sync_loopbacks
+                    o_node._ports[interface_id] = {"category": "loopback"}
+                IndexError: list assignment index out of range
+                None
+                """
+                # don't copy to self or phy as already there
+                continue
+
+            o_node = overlay.node(self)
+            if interface_id in o_node._ports:
+                # something has gone wrong - allocated the id over the top
+                # shouldn't happen since the next free id is from the phy
+
+                log.warning("Internal consistency error with copying loopback "
+                    "interface %s to %s in %s", interface_id, self, overlay)
+                continue
+
+            o_node._ports[interface_id] = {"category": "loopback"}
+
     def add_loopback(self, *args, **kwargs):
         '''Public function to add a loopback interface'''
 
         interface_id = self._add_interface(category='loopback', *args,
                                            **kwargs)
+
+        #TODO: want to add loopbacks to all overlays the node is in
+        self._sync_loopbacks(interface_id)
         return NmPort(self.anm, self.overlay_id,
-                                 self.node_id, interface_id)
+                      self.node_id, interface_id)
 
     def add_interface(self, *args, **kwargs):
         """Public function to add interface"""
 
         interface_id = self._add_interface(*args, **kwargs)
         return NmPort(self.anm, self.overlay_id,
-                                 self.node_id, interface_id)
+                      self.node_id, interface_id)
 
     def interfaces(self, *args, **kwargs):
         """Public function to view interfaces"""
@@ -186,14 +293,14 @@ class NmNode(object):
 
             return all(getattr(interface, key) for key in args) \
                 and all(getattr(interface, key) == val for (key,
-                        val) in kwargs.items())
+                                                            val) in kwargs.items())
 
         all_interfaces = iter(NmPort(self.anm,
-                              self.overlay_id, self.node_id,
-                              interface_id) for interface_id in
+                                     self.overlay_id, self.node_id,
+                                     interface_id) for interface_id in
                               self._interface_ids())
 
-        retval = (i for i in all_interfaces if filter_func(i))
+        retval = [i for i in all_interfaces if filter_func(i)]
         return retval
 
     def interface(self, key):
@@ -202,7 +309,7 @@ class NmNode(object):
         try:
             if key.interface_id in self._interface_ids():
                 return NmPort(self.anm, self.overlay_id,
-                                         self.node_id, key.interface_id)
+                              self.node_id, key.interface_id)
         except AttributeError:
 
             # try with key as id
@@ -210,7 +317,7 @@ class NmNode(object):
             try:
                 if key in self._interface_ids():
                     return NmPort(self.anm, self.overlay_id,
-                                             self.node_id, key)
+                                  self.node_id, key)
             except AttributeError:
 
                 # no match for either
@@ -228,7 +335,8 @@ class NmNode(object):
 
     def _interface_ids(self):
         """Returns interface ids for this node"""
-        #TODO: use from this layer, otherwise can get errors iterating when eg vrfs
+        # TODO: use from this layer, otherwise can get errors iterating when eg
+        # vrfs
         return self._ports.keys()
 
         if self.overlay_id != 'phy' and self['phy']:
@@ -237,7 +345,7 @@ class NmNode(object):
             # the interface mappings from phy
 
             return self['phy']._graph.node[self.node_id]['_ports'
-                                                      ].keys()
+                                                         ].keys()
         else:
             try:
                 return self._ports.keys()
@@ -263,7 +371,7 @@ class NmNode(object):
             return self.anm.overlay_nx_graphs[self.overlay_id]
         except Exception, e:
             log.warning("Error accessing overlay %s for node %s: %s" %
-                (self.overlay_id, self.node_id, e))
+                        (self.overlay_id, self.node_id, e))
 
     @property
     def _nx_node_data(self):
@@ -272,10 +380,24 @@ class NmNode(object):
             return self._graph.node[self.node_id]
         except Exception, e:
             log.warning("Error accessing node data %s for node %s: %s" %
-                (self.overlay_id, self.node_id, e))
+                        (self.overlay_id, self.node_id, e))
 
     def is_router(self):
-        """Either from this graph or the physical graph"""
+        """Either from this graph or the physical graph
+
+        >>> anm = autonetkit.topos.mixed()
+        >>> r1 = anm['phy'].node("r1")
+        >>> r1.is_router()
+        True
+        >>> s1 = anm['phy'].node("s1")
+        >>> s1.is_router()
+        False
+        >>> sw1 = anm['phy'].node("sw1")
+        >>> sw1.is_router()
+        False
+
+        """
+        #self.log_info("add int")
 
         return self.device_type == 'router' or self['phy'].device_type \
             == 'router'
@@ -288,13 +410,39 @@ class NmNode(object):
             == device_type
 
     def is_switch(self):
-        """Returns if device is a switch"""
+        """Returns if device is a switch
+
+        >>> anm = autonetkit.topos.mixed()
+        >>> r1 = anm['phy'].node("r1")
+        >>> r1.is_switch()
+        False
+        >>> s1 = anm['phy'].node("s1")
+        >>> s1.is_switch()
+        False
+        >>> sw1 = anm['phy'].node("sw1")
+        >>> sw1.is_switch()
+        True
+
+        """
 
         return self.device_type == 'switch' or self['phy'].device_type \
             == 'switch'
 
     def is_server(self):
-        """Returns if device is a server"""
+        """Returns if device is a server
+
+        >>> anm = autonetkit.topos.mixed()
+        >>> r1 = anm['phy'].node("r1")
+        >>> r1.is_server()
+        False
+        >>> s1 = anm['phy'].node("s1")
+        >>> s1.is_server()
+        True
+        >>> sw1 = anm['phy'].node("sw1")
+        >>> sw1.is_server()
+        False
+
+        """
 
         return self.device_type == 'server' or self['phy'].device_type \
             == 'server'
@@ -302,6 +450,18 @@ class NmNode(object):
     def is_l3device(self):
         """Layer 3 devices: router, server, cloud, host
         ie not switch
+
+        >>> anm = autonetkit.topos.mixed()
+        >>> r1 = anm['phy'].node("r1")
+        >>> r1.is_l3device()
+        True
+        >>> s1 = anm['phy'].node("s1")
+        >>> s1.is_l3device()
+        True
+        >>> sw1 = anm['phy'].node("sw1")
+        >>> sw1.is_l3device()
+        False
+
         """
         return self.is_router() or self.is_server()
 
@@ -317,13 +477,23 @@ class NmNode(object):
 
     @raw_interfaces.setter
     def raw_interfaces(self, value):
-     self._ports = value
+        self._ports = value
 
     @property
     def asn(self):
-        """Returns ASN of this node"""
+        """Returns ASN of this node
+
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> r1.asn
+        1
+        >>> r5 = anm['phy'].node("r5")
+        >>> r5.asn
+        2
+
+        """
         # TODO: make a function (not property)
-        #TODO: refactor, for nodes created such as virtual switches
+        # TODO: refactor, for nodes created such as virtual switches
 
         try:
             return self._graph.node[self.node_id]['asn']  # not in this graph
@@ -378,29 +548,68 @@ class NmNode(object):
         return NmGraph(self.anm, self.overlay_id)
 
     def degree(self):
-        """Returns degree of node"""
+        """Returns degree of node
+
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> r1.degree()
+        2
+        >>> r2 = anm['phy'].node("r2")
+        >>> r2.degree()
+        3
+
+        """
+
+        # TODO: add example for multi-edge
 
         return self._graph.degree(self.node_id)
 
-
-
     def neighbors(self, *args, **kwargs):
-        """Returns neighbors of node"""
+        """Returns neighbors of node
+
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> r1.neighbors()
+        [r2, r3]
+        >>> r2 = anm['phy'].node("r2")
+        >>> r2.neighbors()
+        [r4, r1, r3]
+
+        Can also filter
+
+        >>> r2.neighbors(asn=1)
+        [r1, r3]
+        >>> r2.neighbors(asn=2)
+        [r4]
+
+
+        """
 
         neighs = list(NmNode(self.anm, self.overlay_id, node)
-                    for node in self._graph.neighbors(self.node_id))
+                      for node in self._graph.neighbors(self.node_id))
 
         return self._overlay.filter(neighs, *args, **kwargs)
 
     def neighbor_interfaces(self, *args, **kwargs):
+        """
+
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> r1.neighbor_interfaces()
+        [eth0.r2, eth0.r3]
+        >>> r2 = anm['phy'].node("r2")
+        >>> r2.neighbor_interfaces()
+        [eth0.r4, eth0.r1, eth1.r3]
+
+        """
 
         # TODO: implement filtering for args and kwargs
 
         if len(args) or len(kwargs):
             log.warning("Attribute-based filtering not currently",
-             "supported for neighbor_interfaces")
+                        "supported for neighbor_interfaces")
 
-        return iter(edge.dst_int for edge in self.edges())
+        return list(edge.dst_int for edge in self.edges())
 
     @property
     # TODO: make a function to reflect dynamic nature: constructed from other
@@ -417,11 +626,21 @@ class NmNode(object):
         try:
             del data['_ports']
         except KeyError:
-            pass # no interfaces set
+            pass  # no interfaces set
         return str(data)
 
     def edges(self, *args, **kwargs):
-        """Edges to/from this node"""
+        """Edges to/from this node
+
+        >>> anm = autonetkit.topos.house()
+        >>> r1 = anm['phy'].node("r1")
+        >>> r1.edges()
+        [phy: (r1, r2), phy: (r1, r3)]
+        >>> r2 = anm['phy'].node("r2")
+        >>> r2.edges()
+        [phy: (r2, r4), phy: (r2, r1), phy: (r2, r3)]
+
+        """
 
         return list(self._overlay.edges(self, *args, **kwargs))
 
@@ -445,7 +664,7 @@ class NmNode(object):
     def __getattr__(self, key):
         """Returns node property
         This is useful for accesing attributes passed through from graphml"""
-        #TODO: refactor/document this logic
+        # TODO: refactor/document this logic
 
         try:
             node_data = self._graph.node[self.node_id]
@@ -463,15 +682,17 @@ class NmNode(object):
                 result = node_data[key]
                 return result
             except KeyError:
+                #TODO: check if in self['phy'[ here]
+
                 if key == "device_type":
                     return self['phy'].device_type
                 if key == "device_subtype":
                     return self['phy'].device_subtype
 
                 # from http://stackoverflow.com/q/2654113
-                self.log.debug(
-                    "Accessing unset attribute %s in %s" % (key,
-                        self.overlay_id))
+                #self.log.debug(
+                    #"Accessing unset attribute %s in %s" % (key,
+                                                            #self.overlay_id))
                 return
 
         # map through to phy
