@@ -21,7 +21,8 @@ class RouterCompiler(DeviceCompiler):
 
     def ibgp_session_data(self, session, ip_version):
 
-        # Don't make staticmethod as may want to extend (eDeviceCompiler.g. in IOS compiler for vpnv4)
+        # Don't make staticmethod as may want to extend (DeviceCompiler.g. in
+        # IOS compiler for vpnv4)
 
         node = session.src
         neigh = session.dst
@@ -35,7 +36,7 @@ class RouterCompiler(DeviceCompiler):
             use_ipv4 = False
             use_ipv6 = True
 
-        #TODO: return ConfigStanza rather than a dict
+        # TODO: return ConfigStanza rather than a dict
         data = {  # TODO: this is platform dependent???
             'neighbor': neigh.label,
             'use_ipv4': use_ipv4,
@@ -43,7 +44,7 @@ class RouterCompiler(DeviceCompiler):
             'asn': neigh.asn,
             'loopback': neigh_ip.loopback,
             'update_source': node.loopback_zero.id,
-            }
+        }
         return data
 
     def ebgp_session_data(self, session, ip_version):
@@ -65,7 +66,7 @@ class RouterCompiler(DeviceCompiler):
         md5_password = session.md5_password
         multihop = session.multihop
 
-        #TODO: return ConfigStanza rather than a dict
+        # TODO: return ConfigStanza rather than a dict
         data = {  # TODO: change templates to access from node.bgp.lo_int
             'neighbor': neigh.label,
             'use_ipv4': use_ipv4,
@@ -77,7 +78,7 @@ class RouterCompiler(DeviceCompiler):
             'update_source': node.loopback_zero.id,
             'md5_password': md5_password,
             'multihop': multihop,
-            }
+        }
         return data
 
     def __init__(self, nidb, anm):
@@ -86,7 +87,7 @@ class RouterCompiler(DeviceCompiler):
         super(RouterCompiler, self).__init__(nidb, anm)
 
     def compile(self, node):
-        node.do_render = True # turn on rendering
+        node.do_render = True  # turn on rendering
 
         phy_node = self.anm['phy'].node(node)
         ipv4_node = self.anm['ipv4'].node(node)
@@ -97,12 +98,12 @@ class RouterCompiler(DeviceCompiler):
         node.ip.use_ipv4 = phy_node.use_ipv4 or False
         node.ip.use_ipv6 = phy_node.use_ipv6 or False
         if not (node.ip.use_ipv4 and node.ip.use_ipv6):
-            node.log.debug('Neither IPv4 nor IPv6 specified: using default IPv4')
-
+            node.log.debug(
+                'Neither IPv4 nor IPv6 specified: using default IPv4')
 
         self.static_routes(node)
 
-            # node.ip.use_ipv4 = True
+        # node.ip.use_ipv4 = True
 
         node.label = naming.network_hostname(phy_node)
         node.input_label = phy_node.id
@@ -112,7 +113,8 @@ class RouterCompiler(DeviceCompiler):
             node.loopback_subnet.prefixlen = 32
 
         if self.anm['phy'].data.enable_routing:
-            node.router_id = ipv4_node.loopback  # applies even if ipv4 disabled, used for eg eigrp, bgp, ...
+            # applies even if ipv4 disabled, used for eg eigrp, bgp, ...
+            node.router_id = ipv4_node.loopback
 
         self.interfaces(node)
         if self.anm.has_overlay('ospf') and node in self.anm['ospf']:
@@ -121,14 +123,14 @@ class RouterCompiler(DeviceCompiler):
             self.isis(node)
         if self.anm.has_overlay('eigrp') and node in self.anm['eigrp']:
             self.eigrp(node)
-        #TODO: drop bgp overlay
+        # TODO: drop bgp overlay
         bgp_overlays = ["bgp", "ebgp_v4", "ibgp_v4", "ebgp_v6", "ibgp_v6"]
         use_bgp = False
         for overlay in bgp_overlays:
             if (self.anm.has_overlay(overlay)
-                and node in self.anm[overlay]
-                and self.anm[overlay].node(node).degree() > 0
-                ):
+                        and node in self.anm[overlay]
+                        and self.anm[overlay].node(node).degree() > 0
+                    ):
                 use_bgp = True
                 break
 
@@ -141,11 +143,11 @@ class RouterCompiler(DeviceCompiler):
         if ipv4_node and ipv4_node.static_routes:
             for route in ipv4_node.static_routes:
                 stanza = ConfigStanza(
-                    prefix = route.prefix,
-                    netmask = route.netmask,
-                    nexthop = route.nexthop,
-                    metric = route.metric,
-                    )
+                    prefix=route.prefix,
+                    netmask=route.netmask,
+                    nexthop=route.nexthop,
+                    metric=route.metric,
+                )
                 node.ipv4_static_routes.append(stanza)
 
         node.ipv6_static_routes = []
@@ -153,10 +155,10 @@ class RouterCompiler(DeviceCompiler):
         if ipv6_node and ipv6_node.static_routes:
             for route in ipv6_node.static_routes:
                 stanza = ConfigStanza(
-                    prefix = route.prefix,
-                    nexthop = route.nexthop,
-                    metric = route.metric,
-                    )
+                    prefix=route.prefix,
+                    nexthop=route.nexthop,
+                    metric=route.metric,
+                )
                 node.ipv6_static_routes.append(stanza)
 
     def interfaces(self, node):
@@ -172,8 +174,8 @@ class RouterCompiler(DeviceCompiler):
             node.loopback_zero.ipv4_address = ipv4_node.loopback
             node.loopback_zero.ipv4_subnet = node.loopback_subnet
 
-        #TODO: bne consistent wit hcidr name so can use in cisco ios xr templates
-        #if node.ip.use_ipv6:
+        # TODO: bne consistent wit hcidr name so can use in cisco ios xr templates
+        # if node.ip.use_ipv6:
             #ipv6_node = self.anm['ipv6'].node(node)
             #node.loopback_zero.ipv6_address = ipv6_node.loopback
             #node.loopback_zero.ipv6_subnet = node.loopback_subnet
@@ -185,81 +187,13 @@ class RouterCompiler(DeviceCompiler):
             # TODO: allocate ID in platform compiler
 
             if not phy_int:
-                # for instance if added as management interface to nidb in compile
+                # for instance if added as management interface to nidb in
+                # compile
                 continue
 
             interface.custom_config = phy_int.custom_config
 
             interface.description = phy_int.description
-            remote_edges = phy_int.edges()
-            if len(remote_edges):
-                interface.description = 'to %s' \
-                    % remote_edges[0].dst.label
-
-            # TODO: fix the description to use mapped label
-
-            if node.ip.use_ipv4:
-                ipv4_int = phy_int['ipv4']
-                if ipv4_int.is_bound:
-
-                    # interface is connected
-
-                    interface.use_ipv4 = True
-                    interface.ipv4_address = ipv4_int.ip_address
-                    interface.ipv4_subnet = ipv4_int.subnet
-                    interface.ipv4_cidr = \
-                        sn_preflen_to_network(interface.ipv4_address,
-                            interface.ipv4_subnet.prefixlen)
-
-            if node.ip.use_ipv6:
-                ipv6_int = phy_int['ipv6']
-                if ipv6_int.is_bound:
-
-                    # interface is connected
-
-                    interface.use_ipv6 = True
-
-# TODO: for consistency, make ipv6_cidr
-
-                    interface.ipv6_subnet = ipv6_int.subnet
-                    try:
-                        interface.ipv6_address = \
-                        sn_preflen_to_network(ipv6_int.ip_address,
-                            interface.ipv6_subnet.prefixlen)
-                    except AttributeError:
-                        log.warning("Unable to format interface ")
-
-        for interface in node.loopback_interfaces():
-
-            # TODO: check if nonzero is different to __eq__
-
-            if interface == node.loopback_zero:
-                continue
-            else:
-                phy_int = self.anm['phy'].interface(interface)
-                if node.ip.use_ipv4:
-                    ipv4_int = phy_int['ipv4']
-                    interface.use_ipv4 = True
-
-                    interface.ipv4_address = ipv4_int.loopback
-                    interface.ipv4_subnet = node.loopback_subnet
-                    interface.ipv4_cidr = \
-                        sn_preflen_to_network(interface.ipv4_address,
-                            interface.ipv4_subnet.prefixlen)
-
-                if node.ip.use_ipv6:
-                    ipv6_int = phy_int['ipv6']
-                    interface.use_ipv6 = True
-
-# TODO: for consistency, make ipv6_cidr
-                    # interface.ipv6_subnet = ipv6_int.loopback # TODO: do we need for consistency?
-
-                    interface.ipv6_address = \
-                        sn_preflen_to_network(ipv6_int.loopback, 128)
-
-                # secondary loopbacks
-                # TODO: check why vrf names not showing up for all
-                # print vrf_interface.vrf_name
 
             continue
 
@@ -274,7 +208,8 @@ class RouterCompiler(DeviceCompiler):
 
         ospf_stanza.custom_config = ospf_node.custom_config
 
-        node.ospf.ipv4_mpls_te = False  # default, inherited enable if necessary
+        # default, inherited enable if necessary
+        node.ospf.ipv4_mpls_te = False
 
         node.ospf.loopback_area = g_ospf.node(node).area or 0
 
@@ -294,11 +229,12 @@ class RouterCompiler(DeviceCompiler):
             ospf_int = g_ospf.interface(interface)
             if ospf_int and ospf_int.is_bound:
                 area = ospf_int.area
-                #TODO: can we remove the next line?
+                # TODO: can we remove the next line?
                 area = str(area)  # can't serialize IPAddress object to JSON
-                #TODO: put in interface rather than interface.id for consistency
-                stanza = ConfigStanza(id = interface.id,
-                        cost = int(ospf_int.cost), passive = False)
+                # TODO: put in interface rather than interface.id for
+                # consistency
+                stanza = ConfigStanza(id=interface.id,
+                                      cost=int(ospf_int.cost), passive=False)
 
                 if node.ip.use_ipv4:
                     stanza.ipv4_address = ospf_int['ipv4'].ip_address
@@ -312,17 +248,21 @@ class RouterCompiler(DeviceCompiler):
         loopback_zero = node.loopback_zero
         ospf_loopback_zero = g_ospf.interface(loopback_zero)
         router_area = ospf_loopback_zero.area  # area assigned to router
-        router_area = str(router_area)  # can't serialize IPAddress object to JSON
-        stanza = ConfigStanza(id = node.loopback_zero.id,
-            cost = 0, passive = True)
+        # can't serialize IPAddress object to JSON
+        router_area = str(router_area)
+        stanza = ConfigStanza(id=node.loopback_zero.id,
+                              cost=0, passive=True)
         interfaces_by_area[router_area].append(stanza)
 
         node.ospf.interfaces_by_area = ConfigStanza(**interfaces_by_area)
 
+        # TODO: split this into a generic IGP function
         added_networks = set()
         for interface in node.physical_interfaces():
             if interface.exclude_igp:
                 continue  # don't configure IGP for this interface
+            if not interface.use_ipv4:
+                continue
             ipv4_int = g_ipv4.interface(interface)
             ospf_int = g_ospf.interface(interface)
             if not ospf_int.is_bound:
@@ -333,20 +273,45 @@ class RouterCompiler(DeviceCompiler):
                 try:
                     ospf_cost = netaddr.IPAddress(ospf_int.cost)
                 except (TypeError, netaddr.AddrFormatError):
-                    log.debug('Using default OSPF cost of 1 for %s on %s'
-                               % (ospf_int, node))
+                    log.debug('Using default OSPF cost of 1 for %s on %s',
+                              ospf_int, node)
                     ospf_cost = 1  # default
             interface.ospf_cost = ospf_cost
             network = ipv4_int.subnet
 
+            #TODO: refactor to allow injecting loopback IPs, etc into IGP
             if ospf_int and ospf_int.is_bound and network \
-                not in added_networks:  # don't add more than once
+                    not in added_networks:  # don't add more than once
                 added_networks.add(network)
-                link_stanza = ConfigStanza(network = network, interface = interface, area = ospf_int.area)
+                link_stanza = ConfigStanza(
+                    network=network, interface=interface, area=ospf_int.area)
                 node.ospf.ospf_links.append(link_stanza)
 
-            # also add networks for subnets to servers in the same AS
+        for interface in node.loopback_interfaces():
+            phy_int = self.anm['phy'].interface(interface)
+            ipv4_int = g_ipv4.interface(interface)
+            if not phy_int.inject_to_igp:
+                #TODO: need to confirm which area to use
+                continue
 
+            network = ipv4_int.subnet
+            if network in added_networks:
+                #TODO: may want to warn here
+                continue # already advertised ("how? - warn if so!)
+
+            # Use the same area as Loopback Zero
+            area = node.ospf.loopback_area
+
+            if not network:
+                log.info("Not injecting unset network on loopback %s "
+                    "to IGP", interface)
+                continue
+
+            link_stanza = ConfigStanza(
+             network=network, interface=interface, area=area)
+            node.ospf.ospf_links.append(link_stanza)
+
+            # also add networks for subnets to servers in the same AS
 
     def bgp(self, node):
         phy_node = self.anm['phy'].node(node)
@@ -361,7 +326,8 @@ class RouterCompiler(DeviceCompiler):
         node.bgp.ipv4_advertise_subnets = []
         if node.ip.use_ipv4:
             node.bgp.ipv4_advertise_subnets = \
-                g_ipv4.data.infra_blocks.get(asn) or []  # could be none (if one-node AS) default empty list
+                g_ipv4.data.infra_blocks.get(
+                    asn) or []  # could be none (one-node AS) default empty list
         node.bgp.ipv6_advertise_subnets = []
         if node.ip.use_ipv6:
             g_ipv6 = self.anm['ipv6']
@@ -375,8 +341,9 @@ class RouterCompiler(DeviceCompiler):
         g_ibgp_v4 = self.anm['ibgp_v4']
         for session in sort_sessions(g_ibgp_v4.edges(phy_node)):
             if session.exclude:
-                log.debug('Skipping excluded ibgp session %s' % session)
-                continue  # exclude from regular ibgp config (eg VRF, VPLS, etc)
+                log.debug('Skipping excluded ibgp session %s', session)
+                # exclude from regular ibgp config (eg VRF, VPLS, etc)
+                continue
 
             data = self.ibgp_session_data(session, ip_version=4)
             bgp_stanza = ConfigStanza(**data)
@@ -395,8 +362,9 @@ class RouterCompiler(DeviceCompiler):
             g_ibgp_v6 = self.anm['ibgp_v6']
             for session in sort_sessions(g_ibgp_v6.edges(phy_node)):
                 if session.exclude:
-                    log.debug('Skipping excluded ibgp session %s' % session)
-                    continue  # exclude from regular ibgp config (eg VRF, VPLS, etc)
+                    log.debug('Skipping excluded ibgp session %s', session)
+                    # exclude from regular ibgp config (eg VRF, VPLS, etc)
+                    continue
                 data = self.ibgp_session_data(session, ip_version=6)
                 bgp_stanza = ConfigStanza(**data)
 
@@ -420,8 +388,9 @@ class RouterCompiler(DeviceCompiler):
         g_ebgp_v4 = self.anm['ebgp_v4']
         for session in sort_sessions(g_ebgp_v4.edges(phy_node)):
             if session.exclude:
-                log.debug('Skipping excluded ebgp session %s' % session)
-                continue  # exclude from regular ibgp config (eg VRF, VPLS, etc)
+                log.debug('Skipping excluded ebgp session %s', session)
+                # exclude from regular ibgp config (eg VRF, VPLS, etc)
+                continue
             data = self.ebgp_session_data(session, ip_version=4)
             bgp_stanza = ConfigStanza(**data)
             ebgp_neighbors.append(bgp_stanza)
@@ -430,13 +399,14 @@ class RouterCompiler(DeviceCompiler):
             g_ebgp_v6 = self.anm['ebgp_v6']
             for session in sort_sessions(g_ebgp_v6.edges(phy_node)):
                 if session.exclude:
-                    log.debug('Skipping excluded ebgp session %s' % session)
-                    continue  # exclude from regular ibgp config (eg VRF, VPLS, etc)
+                    log.debug('Skipping excluded ebgp session %s', session)
+                    # exclude from regular ibgp config (eg VRF, VPLS, etc)
+                    continue
                 data = self.ebgp_session_data(session, ip_version=6)
                 bgp_stanza = ConfigStanza(**data)
                 ebgp_neighbors.append(bgp_stanza)
 
-        ebgp_neighbors = sorted(ebgp_neighbors, key = lambda x: x.asn)
+        ebgp_neighbors = sorted(ebgp_neighbors, key=lambda x: x.asn)
         node.bgp.ebgp_neighbors = ebgp_neighbors
 
         return
@@ -458,7 +428,7 @@ class RouterCompiler(DeviceCompiler):
             if not eigrp_int.is_bound:
                 continue  # not an EIGRP interface
             network = ipv4_int.subnet
-            if eigrp_int and eigrp_int.is_bound:
+            if eigrp_int and eigrp_int.is_bound and interface.use_ipv4:
                 ipv4_networks.add(network)
 
         # Loopback zero subnet
@@ -473,7 +443,8 @@ class RouterCompiler(DeviceCompiler):
         process_id = isis_node.process_id
         node.isis.custom_config = isis_node.custom_config
 
-        node.isis.ipv4_mpls_te = False  # default, inherited enable if necessary
+        # default, inherited enable if necessary
+        node.isis.ipv4_mpls_te = False
 
         for interface in node.physical_interfaces():
             if interface.exclude_igp:
@@ -490,9 +461,9 @@ class RouterCompiler(DeviceCompiler):
                     'use_ipv4': node.ip.use_ipv4,
                     'use_ipv6': node.ip.use_ipv6,
                     'multipoint': isis_int.multipoint,
-                    }
+                }
 
-                          # TODO: add wrapper for this
+                # TODO: add wrapper for this
 
         g_isis = self.anm['isis']
         isis_node = self.anm['isis'].node(node)
@@ -508,5 +479,4 @@ class RouterCompiler(DeviceCompiler):
         node.loopback_zero.isis = {'use_ipv4': node.ip.use_ipv4,
                                    'use_ipv6': node.ip.use_ipv6}
 
-
-                          # TODO: add wrapper for this
+        # TODO: add wrapper for this
